<?php

module_load_include('inc', 'update', 'update.manager');

function _moduleinfo_collector_process_helper_modules_fetch_remote(&$context) {
  $result = moduleinfo_collector_api_modules_find(array(
    array('processed', MODULEINFO_COLLECTOR_PROCESS_STEP_3),
    array('status', 1)
  ), 10, 0, TRUE);

  // Filters
  $filter_version = trim(variable_get('moduleinfo_collector_filter_modules_known_without_version', ''));
  if (!empty($filter_version)) {
    $filter_version = explode("\n", $filter_version);
    $filter_version = array_map('trim', $filter_version);
  }

  foreach ($result as $module) {
    if ($module->name == 'drupal') continue;
    if (in_array($module->name, $filter_version)) continue;

    $module->attempts = intval($module->attempts);
    $module->attempts++;

    try {
      $version = _moduleinfo_collector_process_helper_find_fetch_url($module);
    }
    catch (Exception $e) {
      $download_version = variable_get('moduleinfo_collector_module_release_type', 'stable_dev');

      moduleinfo_collector_api_modules_update($module->mid, array(
        'message' => t('No version found to download. Your setting is: !setting', array(
          '!setting' => $download_version,
        )),
        'status' => 0,
        'processed' => $module->processed | MODULEINFO_COLLECTOR_PROCESS_ABORTED,
      ));
      continue;
    }

    $file = _moduleinfo_collector_update_manager_file_get($version->href);

    if ($file) {
      moduleinfo_collector_api_modules_update($module->mid, array(
        'processed' => $module->processed | MODULEINFO_COLLECTOR_PROCESS_FETCH_REMOTE,
        'attempts' => 0,
      ));
    }
    else {
      moduleinfo_collector_api_modules_update($module->mid, array(
        'message' => t('It was not possible to download the module from the page !page after !attempts attempts.', array(
          '!page' => $version->href,
          '!attempts' => $module->attempts,
        )),
        'attempts' => $module->attempts,
      ));

      // TODO: Check if a certain maximum of attempts is hit on that module and set a status, so that it will not be fetched anymore.
      if ($module->attempts >= 5) {
        moduleinfo_collector_api_modules_update($module->mid, array(
          'status' => 0,
          'processed' => $module->processed | MODULEINFO_COLLECTOR_PROCESS_ABORTED,
        ));
      }
    }

    $context['sandbox']['progress']++;
    $context['sandbox']['current_node'] = $module->name;
    $context['message'] = t('Last downloaded module: %module // Processed: %processed of %max', array(
      '%module' => $module->human_name,
      '%processed' => $context['sandbox']['progress'],
      '%max' => $context['sandbox']['max'])
    );
  }
}

function _moduleinfo_collector_process_helper_find_fetch_url($module) {
  $download_version = variable_get('moduleinfo_collector_module_release_type', 'stable_dev');

  $versions = $module->versions;

  if (!is_array($versions)) {
    throw new Exception("No versions were found to download.");
  }

  if (!count($versions)) {
    throw new Exception("No versions were found to download.");
  }

  $versions_tmp = array();
  foreach ($versions as $version) {
    switch ($download_version) {
      case 'dev':
        if ($version->type == 'dev' && !empty($version->href)) {
          return $version;
        }
        break;

      case 'stable':
        if ($version->type == 'stable' && !empty($version->href)) {
          return $version;
        }
        break;

      case 'stable_dev':
        if (in_array($version->type, array('dev', 'stable')) && !empty($version->href)) {
          $versions_tmp[$version->type] = $version;
        }
        break;

      case 'usage':
        $versions_tmp[$version->usage] = $version;
        break;
    }
  }

  switch ($download_version) {
    case 'stable_dev':
      if (!count($versions_tmp)) {
        throw new Exception("No stable or dev release were found");
      }

      if (isset($versions_tmp['stable'])) {
        return $versions_tmp['stable'];
      }

      return $versions_tmp['dev'];

    case 'stable':
      throw new Exception("No stable release were found.");

    case 'dev':
      throw new Exception("No dev release were found.");

    case 'usage':
      if (!count($versions_tmp)) {
        throw new Exception("No version was found.");
      }

      $usages = array_keys($versions_tmp);
      return $versions_tmp[max($usages)];
  }
}

function _moduleinfo_collector_update_manager_file_get($url) {
  $parsed_url = parse_url($url);
  $remote_schemes = array('http', 'https', 'ftp', 'ftps', 'smb', 'nfs');
  if (!in_array($parsed_url['scheme'], $remote_schemes)) {
    // This is a local file, just return the path.
    return drupal_realpath($url);
  }

  // Check the cache and download the file if needed.
  $cache_directory = MODULEINFO_COLLECTOR_CACHE_PATH_SCHEMA . MODULEINFO_COLLECTOR_CACHE_PATH_ARCHIVES;
  $local = $cache_directory . '/' . drupal_basename($parsed_url['path']);

  if (!file_exists($local)) {
    return system_retrieve_file($url, $local, FALSE, FILE_EXISTS_REPLACE);
  }
  else {
    return $local;
  }
}