<?php

module_load_include('inc', 'update', 'update.manager');

function _moduleinfo_collector_process_helper_modules_extract(&$context) {
  $modules = moduleinfo_collector_api_modules_find(array(
    array('processed', 163), // TODO
    array('status', 1)
  ), 10, 0, TRUE);

  foreach ($modules as $module) {
    if (!isset($module->versions)) {
      continue;
    }

    $version_found = FALSE;
    foreach ($module->versions as $version) {
      if ($version->downloaded == 0) {
        continue;
      }

      $version_found = TRUE;

      $filename = basename($version->href);

      $path = MODULEINFO_COLLECTOR_CACHE_PATH_SCHEMA . MODULEINFO_COLLECTOR_CACHE_PATH_ARCHIVES . '/' . $filename;
      $destination = MODULEINFO_COLLECTOR_CACHE_PATH_SCHEMA .  MODULEINFO_COLLECTOR_CACHE_PATH_MODULES;

      $archive = update_manager_archive_extract($path, $destination);

      // Each downloaded module can come with a lot of submodules.
      $modules_parsed = _moduleinfo_collector_system_rebuild_module_data();

      if (!count($modules_parsed)) {
        moduleinfo_collector_api_modules_update($module->mid, array(
          'message' => t('No modules found in the archive !archive.', array(
            '!archive' => $filename,
          )),
          'status' => 0,
        ));

        $path_delete = MODULEINFO_COLLECTOR_CACHE_PATH_SCHEMA . MODULEINFO_COLLECTOR_CACHE_PATH_MODULES;

        file_unmanaged_delete_recursive($path_delete);
        file_prepare_directory($path_delete, FILE_CREATE_DIRECTORY);
        break;
      }

      foreach ($modules_parsed as $name => $parsed) {
        if (preg_match('/.+_test$/', $name)) {
          unset($modules_parsed[$name]);
          continue;
        }

        // Update informations on the Module
        $mid = $module->mid;

        if ($name == $module->name) {
          moduleinfo_collector_api_modules_update($module->mid, array(
            'human_name' => $parsed->info['name'],
            'description' => $parsed->info['description'],
            'project' => strtolower($parsed->info['project']),
          ));
        }
        else {
          // Add a child module to the table,
          $data = array(
            'name' => $name,
            'human_name' => $parsed->info['name'],
            'description' => $parsed->info['description'],
            'module_parent' => $module->name,
            'project' => strtolower($parsed->info['project'])
          );

          $mid = moduleinfo_collector_api_modules_insertreplace($data);
        }

        if (isset($parsed->info['dependencies']) && count($parsed->info['dependencies'])) {
          foreach ($parsed->info['dependencies'] as $dependency) {
            moduleinfo_collector_api_dependency_insert($mid, array(
              'name' => $dependency,
            ));
          }
        }

        $context_dummy = array();
        $files = _moduleinfo_collect_module_files($module->name, dirname($parsed->filename));

        _moduleinfo_cache_write('files', $files, $module->name, 'moduleinfo_collector');
        _moduleinfo_cache_operations($module->name, 'moduleinfo_collector');
        _moduleinfo_scan_post_process('moduleinfo_collector', $context_dummy);

        moduleinfo_collector_api_modules_update($module->mid, array(
          'processed' => $module->processed | MODULEINFO_COLLECTOR_PROCESS_MODULES_EXTRACT,
          'attempts' => 0,
        ));

        $path_delete = MODULEINFO_COLLECTOR_CACHE_PATH_SCHEMA . MODULEINFO_COLLECTOR_CACHE_PATH_MODULES;

        file_unmanaged_delete_recursive($path_delete);
        file_prepare_directory($path_delete, FILE_CREATE_DIRECTORY);
      }
    }

    if (!$version_found) {
      moduleinfo_collector_api_modules_update($module->mid, array(
        'message' => t('It seems that there was no version downloaded for this module in a previous step.'),
        'status' => 0,
      ));
    }

    $context['sandbox']['progress']++;
    $context['sandbox']['current_node'] = $module->name;
    $context['message'] = t('Last downloaded module: %module // Processed: %processed of %max', array(
      '%module' => $module->human_name,
      '%processed' => $context['sandbox']['progress'],
      '%max' => $context['sandbox']['max'])
    );
  }
}
