<?php

function _moduleinfo_collector_process_helper_project_page(&$context) {
  $result = moduleinfo_collector_api_modules_find(array(
    array('processed', MODULEINFO_COLLECTOR_PROCESS_STEP_2),
    array('status', 1)
  ));

  $qp_options = array('strip_low_ascii' => TRUE, 'convert_to_encoding' => 'UTF-8', 'convert_from_encoding' => 'UTF-8', 'replace_entities' => TRUE);

  // Filters
  $filter_version = trim(variable_get('moduleinfo_collector_filter_modules_known_without_version', ''));
  if (!empty($filter_version)) {
    $filter_version = explode("\n", $filter_version);
    $filter_version = array_map('trim', $filter_version);
  }

  foreach ($result as $module) {
    if ($module->name == 'drupal') continue;
    if (in_array($module->name, $filter_version)) continue;

    $module->attempts = intval($module->attempts);
    $module->attempts++;

    $data = _moduleinfo_collector_request_file($module->href_project);

    if (!$data) {
      moduleinfo_collector_api_modules_update($module->mid, array(
        'message' => t('It was not possible to download data from the page !page after !attempts attempts.', array(
          '!page' => $module->href_project,
          '!attempts' => $module->attempts,
        )),
        'attempts' => $module->attempts,
      ));

      // TODO: Check if a certain maximum of attempts is hit on that module and set a status, so that it will not be fetched anymore.
      if ($module->attempts >= 5) {
        moduleinfo_collector_api_modules_update($module->mid, array(
          'status' => 0,
          'processed' => $module->processed | MODULEINFO_COLLECTOR_PROCESS_ABORTED,
        ));
      }
    }

    $module->attempts = 0;

    $account = htmlqp($data, NULL, $qp_options)
      ->find('div.submitted', NULL, $qp_options)
      ->children('a')
      ->text();

    $terms = htmlqp($data, NULL, $qp_options)
      ->find('span.terms', NULL, $qp_options)
      ->children('a');

    $taxonomy = array();
    foreach ($terms as $term) {
      $taxonomy[] = qp($term)->text();
    }

    $project_infos = htmlqp($data, NULL, $qp_options)
      ->find('div.project-info ul', NULL, $qp_options)
      ->children('li');

    $downloads = 0;
    foreach ($project_infos as $project_info) {
      $info = trim(qp($project_info)->text());

      if (preg_match('/^Downloads: +?([0-9,]+)$/', $info, $hits)) {
        $downloads = intval(preg_replace('/[^0-9]/', '', $hits[1]));
      }
    }

    moduleinfo_collector_api_modules_update($module->mid, array(
      'author' => $account,
      'downloads' => $downloads,
    ));

    moduleinfo_collector_api_taxonomy_insert($module->mid, $taxonomy);

    moduleinfo_collector_api_modules_update($module->mid, array(
      'processed' => $module->processed | MODULEINFO_COLLECTOR_PROCESS_PROJECT_PAGE,
      'attempts' => 0,
    ));

    // Store some result for post-processing in the finished callback.
    $context['results'][] = check_plain($module->name);

    // Update our progress information.
    $context['sandbox']['progress']++;
    $context['sandbox']['current_node'] = $module->name;
    $context['message'] = t('Now processing %module', array('%module' => $module->human_name));
  }
}