<?php

function _moduleinfo_collector_process_helper_versions(&$context) {
  $result = moduleinfo_collector_api_modules_find(array(
    array('processed', MODULEINFO_COLLECTOR_PROCESS_USAGE),
    array('processed', MODULEINFO_COLLECTOR_PROCESS_USAGE | MODULEINFO_COLLECTOR_PROCESS_VERSIONS, '<>'),
    array('status', 1)
  ));

  $qp_options = array('strip_low_ascii' => TRUE, 'convert_to_encoding' => 'UTF-8', 'convert_from_encoding' => 'UTF-8', 'replace_entities' => TRUE);

  // Filters
  $filter_version = trim(variable_get('moduleinfo_collector_filter_modules_known_without_version', ''));
  if (!empty($filter_version)) {
    $filter_version = explode("\n", $filter_version);
    $filter_version = array_map('trim', $filter_version);
  }

  foreach ($result as $module) {
    if ($module->name == 'drupal') {
      moduleinfo_collector_api_modules_update($module->mid, array(
        'message' => t('Drupal will be handled in a seperate step.'),
        'status' => 0,
        'processed' => $module->processed | MODULEINFO_COLLECTOR_PROCESS_ABORTED,
      ));
      continue;
    }

    if (in_array($module->name, $filter_version)) {
      moduleinfo_collector_api_modules_update($module->mid, array(
        'message' => t('This modules was in the list of known modules without a !version version. Therefore skipped.', array(
          '!version' => '7.x',
        )),
        'status' => 0,
        'processed' => $module->processed | MODULEINFO_COLLECTOR_PROCESS_ABORTED,
      ));
      continue;
    }

    $module->attempts = intval($module->attempts);
    $module->attempts++;

    $data = _moduleinfo_collector_request_file($module->href_usage);

    if (!$data) {
      moduleinfo_collector_api_modules_update($module->mid, array(
        'message' => t('It was not possible to download data from the page !page after !attempts attempts.', array(
          '!page' => $module->href_usage,
          '!attempts' => $module->attempts,
        )),
        'attempts' => $module->attempts,
      ));

      // TODO: Check if a certain maximum of attempts is hit on that module and set a status, so that it will not be fetched anymore.
      if ($module->attempts >= 5) {
        moduleinfo_collector_api_modules_update($module->mid, array(
          'status' => 0,
          'processed' => $module->processed | MODULEINFO_COLLECTOR_PROCESS_ABORTED,
        ));
      }
    }

    $module->attempts = 0;

    $qp = htmlqp($data, NULL, $qp_options)
      ->find('#project-usage-project-releases tbody', NULL, $qp_options)
      ->children('tr');

    $versions = array();
    foreach ($qp as $elem) {
      $tds = htmlqp($elem, NULL, $qp_options)->children('td')->get();

      $version = htmlqp($tds[0], NULL, $qp_options)->text();
      $usage = intval(preg_replace('/[^0-9]/', '', htmlqp($tds[1], NULL, $qp_options)->text()));

      // Check if the version is one of the current Drupal branch
      if (!preg_match('/^7.+/', $version, $hits)) {
        continue;
      }

      $versions[$version] = array(
        'version' => $version,
        'usages' => $usage,
        'mid' => $module->mid,
      );
    }

    if (!count($versions)) {
      moduleinfo_collector_api_modules_update($module->mid, array(
        'message' => t('No version found which matches the used drupal !version.', array(
          '!version' => '7.x',
        )),
        'status' => 0,
        'processed' => $module->processed | MODULEINFO_COLLECTOR_PROCESS_ABORTED,
      ));
      continue;
    }

    // Check for the latest version and the dev version.
    _moduleinfo_collector_process_helper_versions_active_finder($module, $versions);

    moduleinfo_collector_api_versions_insert($module->mid, $versions);

    moduleinfo_collector_api_modules_update($module->mid, array(
      'processed' => $module->processed | MODULEINFO_COLLECTOR_PROCESS_VERSIONS,
      'attempts' => 0,
    ));

    // Store some result for post-processing in the finished callback.
    $context['results'][] = check_plain($module->name);

    // Update our progress information.
    $context['sandbox']['progress']++;
    $context['sandbox']['current_node'] = $module->name;
    $context['message'] = t('Now processing %module', array('%module' => $module->human_name));
  }
}

function _moduleinfo_collector_process_helper_versions_active_finder($module, &$versions) {
  // I support only the notation of 7.x-Y.Z and 7.x-Y.Z-dev at this moment.
  // let's find these if possible

  $regex_stable = '/^7.x-(([0-9]+)\\.[0-9]+)$/';
  $regex_dev = '/^7.x-(([0-9]+)\\.x)-dev$/';

  $result_stable = array();
  $result_dev = array();
  foreach ($versions as $i => $version) {
    $versions[$i]['href'] = 'http://ftp.drupal.org/files/projects/' . $module->name . '-' . $version['version'] . '.zip';

    if (preg_match($regex_stable, $version['version'], $hits)) {
      if (!isset($result_stable[$hits[2]])) {
        $result_stable[$hits[2]] = $hits;
        continue;
      }

      $new = floatval($hits[1]);
      $old = floatval($result_stable[$hits[2]][1]);

      if ($new > $old) {
        $result_stable[$hits[2]] = $hits;
      }
    }

    if (preg_match($regex_dev, $version['version'], $hits)) {
      $result_dev[$hits[2]] = $hits;
    }
  }

  if (count($result_stable)) {
    $stables = array_keys($result_stable);

    $stable_version = max($stables);

    $stables = $result_stable[$stable_version];

    $version_full = $stables[0];

    $versions[$version_full]['type'] = 'stable';
  }

  if (count($result_dev)) {
    $devs = array_keys($result_dev);

    $dev_version = max($devs);

    $devs = $result_dev[$dev_version];

    $version_full = $devs[0];

    $versions[$version_full]['type'] = 'dev';
  }
}