<?php

/**
 * Form builder for the update settings tab.
 */
function moduleinfo_collector_settings_form($form) {
  $form = array();

  $form['info'] = array(
    '#markup' => t(
      'To be more accurate on the detailspage, this module scans ALL enabled modules for hooks and more. You can !scan_now and check out the !overview page to have the birdview. Head over to the !modules_page to check what a single module offers to your installation.', array(
        '!scan_now' => l(t('scan your installation now'), 'moduleinfo/scan', array('query' => array('destination' => 'admin/config/development/moduleinfo'))),
        '!overview' => l(t('ModuleInfo overview'), 'admin/reports/moduleinfo'),
        '!modules_page' => l(t('Modules'), 'admin/modules'),
      )
    ),
    '#weight' => -10,
  );

  $form['moduleinfo_collector_filter_modules_exclude'] = array(
    '#type'          => 'textarea',
    '#title'         => t('Which modules should not be processed'),
    '#default_value' => variable_get('moduleinfo_collector_filter_modules_exclude', MODULEINFO_COLLECTOR_FILTER_MODULES_EXCLUDE),
    '#description'   => t('Add one module name (machine readable) per line of modules which shall not be processed.'),
  );

  $form['moduleinfo_collector_filter_modules_known_without_version'] = array(
    '#type'          => 'textarea',
    '#title'         => t('Modules which are known to have no 7.x version, can be added here, so there are not processed.'),
    '#default_value' => variable_get('moduleinfo_collector_filter_modules_known_without_version', file_get_contents(drupal_get_path('module', 'moduleinfo_collector') . '/data/modules_without_7.x_branch.csv')),
    '#description'   => t('These modules were found without a 7.x branch and are presummable blocked from processing. This speed up the process. Check this list or remove it if you don\'t trust it.'),
  );

  $form['moduleinfo_collector_page_modules_list'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Page where to fetch the module list'),
    '#default_value' => variable_get('moduleinfo_collector_page_modules_list', MODULEINFO_COLLECTOR_PAGE_MODULES_LIST),
    '#description'   => t('....'),
  );

  $form['moduleinfo_collector_module_name_regex'] = array(
    '#type'          => 'textfield',
    '#title'         => t('RegEx which will filter a LIST of module names from the source page.'),
    '#default_value' => variable_get('moduleinfo_collector_module_name_regex', MODULEINFO_COLLECTOR_MODULE_NAME_REGEX),
    '#description'   => t('....'),
  );

  $form['moduleinfo_collector_module_release_type'] = array(
    '#type' => 'select',
    '#title' => t('Preferable Module Release Type'),
    '#default_value' => variable_get('moduleinfo_collector_module_release_type', 'latest'),
    '#options' => array(
      'stable' => t('Only stable releases (7.x-Y.Z)'),
      'dev' => t('Only dev releases (7.x-Y.Z-dev)'),
      'usage' => t('Highest usage.'),
      'stable_dev' => t('Prefer stable releases over dev releases.'),
    ),
    '#empty_option' => t('-- Please Choose --'),
    '#description' => t('Which type of release do you prefer.'),
  );

  $form['modules_known'] = array(
    '#type' => 'fieldset',
    '#title' => t('Known Modules'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#tree' => FALSE,
    '#description' => t('To save bandwidth for drupal.org, this will save a list of known modules and an expire date, were this list gets updated.'),
  );

  $foo = variable_get('moduleinfo_collector_cache_modules_known', '');

  $form['modules_known']['moduleinfo_collector_cache_modules_known'] = array(
    '#type'          => 'textarea',
    '#title'         => t('Modules which are known to the moduleinfo collector'),
    '#default_value' => variable_get('moduleinfo_collector_cache_modules_known', ''),
    '#description'   => t('....'),
  );

  $form['modules_known']['moduleinfo_collector_cache_html_files'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Cache downloaded HTML files?'),
    '#default_value' => variable_get('moduleinfo_collector_cache_html_files', '1'),
    '#description'   => t('Mainly for debug reasons, html files can be cached. Speeds things up in reruns.'),
  );

  $form['modules_known']['moduleinfo_collector_cache_modules_known_expires'] = array(
    '#type' => 'date',
    '#title' => t('List expires after'),
    '#default_value' => variable_get('moduleinfo_collector_cache_modules_known_expires', ''),
    '#description' => t('A the list will be ignored and refreshed after this date.'),
  );

  return system_settings_form($form);
}

/**
 * Implements @see hook_form().
 */
function moduleinfo_collector_form($form, &$form_state) {
  $form = array();

  $form['usage'] = array(
    '#type' => 'textfield',
    '#title' => t('Latest minimum usage counter'),
    '#default_value' => '150000',
    '#required' => TRUE,
    '#description' => t('If a module has less than this amount of installations it will not be processed.'),
  );

  $form['age'] = array(
    '#type' => 'select',
    '#title' => t('Age of the module release'),
    '#default_value' => '',
    '#options' => array(
      '2592000' => t('1 month'),
      '7776000' => t('3 month'),
      '15552000' => t('6 month'),
      86400 * 365 => t('1 year'),
      86400 * 365 * 2 => t('2 year'),
      86400 * 365 * 3 => t('3 year'),
      100000000000 => t('no time limit'),
    ),
    '#description' => t('Attention! Don\'t be evil! Do not download older modules as you would accept for your solution.'),
    '#required' => TRUE,
  );

  $form['amount'] = array(
    '#type' => 'select',
    '#title' => t('How many modules to fetch?'),
    '#default_value' => '50',
    '#options' => array(
      '10' => t('Top 10'),
      '50' => t('Top 50'),
      '100' => t('Top 100'),
      '250' => t('Top 250'),
      '1000' => t('Top 1000'),
      '1000000' => t('All'),
    ),
    '#empty_option' => t('-- Please Choose --'),
    '#description' => t('Attention! Don\'t be evil! The amount you put here will a) directly affect your download space. And b) will cause traffic for drupal.org. Don\'t do it to often. In fact do it maximum once a month or so.'),
    '#required' => TRUE,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit')
  );

  return $form;
}

/**
 * Implements
 * @see hook_form_validate().
 */
function moduleinfo_collector_form_validate($form_id, $form_state) {
  $values =& $form_state['values'];

  if (FALSE) {
    form_set_error('', t(''));
  }
}

/**
 * Implements
 * @see hook_form_submit().
 */
function moduleinfo_collector_form_submit($form_id, $form_state) {
  $values =& $form_state['values'];

  $_SESSION['MODULE_COLLECTOR']['settings']['usage'] = $values['usage'];
  $_SESSION['MODULE_COLLECTOR']['settings']['age'] = $values['age'];
  $_SESSION['MODULE_COLLECTOR']['settings']['amount'] = $values['amount'];

  $operations = array();
//  $operations[] = array('_moduleinfo_collector_setup', array());
//  $operations[] = array('_moduleinfo_collector_fetch_usage', array());
//  $operations[] = array('_moduleinfo_collector_fetch_version', array());
//  $operations[] = array('_moduleinfo_collector_fetch_project_page', array());

//  $operations[] = array('_moduleinfo_scan_start', array('moduleinfo_collector'));
//
//  $modules = system_rebuild_module_data();
//
//  foreach ($modules as $name => $module) {
//    if (preg_match('/.+_test/', $name, $hits)) {
//      continue;
//    }
//
//    $operations[] = array('_moduleinfo_scan', array($name, $module, 'moduleinfo_collector'));
//  }
//
//  $operations[] = array('_moduleinfo_scan_post_process', array('moduleinfo_collector'));
// TODO: !!
//  $operations[] = array('_moduleinfo_collector_modules_parse_local', array());

//  $operations[] = array('_moduleinfo_collector_modules_fetch_remote', array());
//  $operations[] = array('_moduleinfo_collector_modules_extract', array());
  $operations[] = array('_moduleinfo_collector_modules_hooks', array());

//  $modules_subdirs = array(
//    '0', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l',
//    'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'
//  );

//  foreach ($modules_subdirs as $module_subdir) {
//    $operations[] = array('_moduleinfo_collector_modules_scan_downloads', array($module_subdir));
//  }

//  $operations[] = array('_moduleinfo_collector_modules_scan_downloads', array('b'));
//  $operations[] = array('_moduleinfo_collector_modules_scan_downloads', array('c'));

  $batch = array(
    'title' => t('Downloading Modules'),
    'operations' => $operations,
    'init_message' => t('Be patiend! This is a very long process.'),
//    'finished' => '_moduleinfo_scan_finish',
    'file' => drupal_get_path('module', 'moduleinfo_collector') . '/moduleinfo_collector.process.inc',
  );

  batch_set($batch);
  batch_process('admin/structure/moduleinfo-collector');
}