<?php

// Modules //

function moduleinfo_collector_api_modules_find($conditions, $range = 10, $offset = 0, $full = FALSE) {
  $qry = db_select('moduleinfo_collector_modules', 'mc')
    ->fields('mc')
    ->range($offset, $range);

  foreach ($conditions as $condition) {
    $condition += array(
      1 => NULL,
      2 => NULL,
    );

    $qry->condition($condition[0], $condition[1], $condition[2]);
  }

  $result = $qry->execute()
    ->fetchAllAssoc('mid');

  if ($full) {
    foreach ($result as $mid => $elem) {
      $versions = moduleinfo_collector_api_versions_find(array(array('mid', $mid)));
      $result[$mid]->versions = $versions;
    }
  }

  return $result;
}

/**
 * Deletes all informations about a certain module.
 *
 * @param $name
 * @return bool
 */
function moduleinfo_collector_api_modules_delete($name = NULL) {
  $mid = NULL;

  if (isset($name)) {
    $result = db_select('moduleinfo_collector_modules', 'm')
      ->fields('m', array('mid'))
      ->condition('name', $name)
      ->execute()
      ->fetchAssoc();

    if (!$result) {
      return FALSE;
    }

    $mid = $result['mid'];
  }
  else {
    db_delete('moduleinfo_collector_taxonomy')
      ->execute();
  }

  $tables = array(
    'moduleinfo_collector_versions',
    'moduleinfo_collector_tags',
    'moduleinfo_collector_hooks',
    'moduleinfo_collector_dependencies',
    'moduleinfo_collector_modules',
  );

  foreach ($tables as $table) {
    $query = db_delete($table);

    if (isset($name) && isset($mid)) {
      $query->condition('mid', $mid);
    }

    $query->execute();
  }
}

function moduleinfo_collector_api_modules_insertreplace($data) {
  $name = $data['name'];

  $data += array(
    'href_project' => '',
    'href_usage' => '',
    'name' => '',
    'author' => '',
    'human_name' => '',
    'description' => '',
    'rank' => 1000000,
    'usages' => 0,
    'position' => 1000000,
    'processed' => 0,
    'created' => time(),
    'changed' => time(),
    'message' => '',
    'local' => 0,
    'status' => 0,
    'downloads' => 0,
    'attempts' => 0,
    'module_parent' => '',
    'project' => '',
  );

  moduleinfo_collector_api_modules_delete($name);

  $return = db_insert('moduleinfo_collector_modules')
    ->fields($data)
    ->execute();

  return $return;
}

function moduleinfo_collector_api_modules_update($mid, $data) {
  db_update('moduleinfo_collector_modules')
    ->fields($data)
    ->condition('mid', $mid)
    ->execute();
}

// Versions //

function moduleinfo_collector_api_versions_insert($mid, $versions) {
  db_delete('moduleinfo_collector_versions')
    ->condition('mid', $mid)
    ->execute();

  foreach ($versions as $version) {
    if (!isset($version['version'])) continue;

    $version += array(
      'mid' => $mid,
      'href' => '',
      'usages' => 0,
      'type' => '',
    );

    db_insert('moduleinfo_collector_versions')
      ->fields($version)
      ->execute();
  }
}


function moduleinfo_collector_api_versions_find($conditions, $range = 10, $offset = 0) {
  $qry = db_select('moduleinfo_collector_versions', 'mc')
    ->fields('mc')
    ->range($offset, $range);

  foreach ($conditions as $condition) {
    $condition += array(
      1 => NULL,
      2 => NULL,
    );

    $qry->condition($condition[0], $condition[1], $condition[2]);
  }

  $result = $qry->execute()
    ->fetchAllAssoc('id');

  return $result;
}

function moduleinfo_collector_api_version_to_process($module) {
  $result = moduleinfo_collector_api_versions_find(array(
    array('mid', $module->mid),
    array('href', '', '<>'),
    array('type', array('stable', 'dev'), 'IN')
  ));

  $settings = variable_get('moduleinfo_collector_module_release_type', 'latest');
}

// Taxonomy //

function moduleinfo_collector_api_taxonomy_insert($mid, $terms) {
  db_delete('moduleinfo_collector_tags')
    ->condition('mid', $mid)
    ->execute();

  foreach ($terms as $term) {
    $test = db_select('moduleinfo_collector_taxonomy', 'mc')
      ->fields('mc')
      ->condition('name', $term)
      ->execute()
      ->fetchAssoc();

    if (!$test) {
      $tid = db_insert('moduleinfo_collector_taxonomy')
        ->fields(array(
          'name' => $term,
        ))
        ->execute();
    }
    else {
      $tid = $test['tid'];
    }

    db_insert('moduleinfo_collector_tags')
      ->fields(array(
        'tid' => $tid,
        'mid' => $mid,
      ))
      ->execute();
  }
}

// Dependency //

function moduleinfo_collector_api_dependency_insert($mid, $data) {
  $data += array(
    'mrid' => 0,
    'name' => '',
    'mid' => $mid,
  );

  if (intval($data['mrid']) > 0 && empty($data['name'])) {
    $result = moduleinfo_collector_api_modules_find(array(
      array('mid', $data['mrid'])
    ));

    if ($result && count($result) == 1) {
      $result = current($result);
      $data['mrid'] = $result->mid;
    }
  }

  if (intval($data['mrid']) == 0 && !empty($data['name'])) {
    $result = moduleinfo_collector_api_modules_find(array(
      array('name', $data['name'])
    ));

    if ($result && count($result) == 1) {
      $result = current($result);
      $data['name'] = $result->name;
    }
  }

  $test = db_select('moduleinfo_collector_dependencies', 'mc')
    ->fields('mc')
    ->condition('name', $data['name'])
    ->execute()
    ->fetchAssoc();

  if (!$test) {
    db_insert('moduleinfo_collector_dependencies')
      ->fields($data)
      ->execute();
  }
}