<?php

include_once 'sites/all/libraries/querypath/src/qp.php';

module_load_include('inc', 'moduleinfo_collector', 'moduleinfo_collector.api');
module_load_include('inc', 'moduleinfo', 'moduleinfo.scan');

function _moduleinfo_collector_setup(&$context) {
  _moduleinfo_collector_cache_setup();
  moduleinfo_collector_api_modules_delete();
}

/**
 * Fetches the http://drupal.org/project/usage list of modules along with
 * useful data to limit the scope of downloads or modules which get processed.
 *
 * STEP 1
 *
 * @param $context
 */
function _moduleinfo_collector_fetch_usage(&$context) {
  module_load_include('inc', 'moduleinfo_collector', 'includes/moduleinfo_collector.usage');

  $url_usage = variable_get('moduleinfo_collector_page_modules_list', MODULEINFO_COLLECTOR_PAGE_MODULES_LIST);

  $data = _moduleinfo_collector_request_file($url_usage);

  if (!$data) {
    throw new Exception(t('It was not possible to download the file: !file', array('!file' => $url_usage)));
  }

  // Check if string is UTF-8
  if (!drupal_validate_utf8($data)) {
    $data = utf8_encode($data);
  }

  $data = strtr($data, 'ÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÐÑÒÓÔÕÖØÙÚÛÜÝÞßàáâãäåæçèéêëìíîïðñòóôõöøùúûýýþÿŔŕ', 'aaaaaaaceeeeiiiidnoooooouuuuybsaaaaaaaceeeeiiiidnoooooouuuyybyRr');

  $qp_options = array('strip_low_ascii' => TRUE, 'convert_to_encoding' => 'UTF-8', 'convert_from_encoding' => 'UTF-8', 'replace_entities' => TRUE);
  $qp = htmlqp($data, NULL, $qp_options)
    ->find('#project-usage-all-projects tbody', NULL, $qp_options)
    ->children('tr');

  $cnt = count($qp);

  if (!isset($context['sandbox']['progress'])) {
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['current_node'] = 0;
    $context['sandbox']['max'] = $cnt;
    $context['sandbox']['next_limit'] = 200;
  }

  _moduleinfo_collector_process_helper_usage($context, $data, $qp);

  // Inform the batch engine that we are not finished,
  // and provide an estimation of the completion level we reached.
  if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
    $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
  }
}

/**
 * STEP 2
 *
 * @param $context
 */
function _moduleinfo_collector_fetch_version(&$context) {
  module_load_include('inc', 'moduleinfo_collector', 'includes/moduleinfo_collector.versions');

  $fetch_settings = $_SESSION['MODULE_COLLECTOR']['settings'];

  if (!isset($context['sandbox']['progress'])) {
    $cnt = db_select('moduleinfo_collector_modules', 'mc')
      ->condition('processed', MODULEINFO_COLLECTOR_PROCESS_STEP_1)
      ->condition('status', 1)
      ->countQuery()
      ->execute()
      ->fetchField();

    $context['sandbox']['progress'] = 0;
    $context['sandbox']['current_node'] = 0;
    $context['sandbox']['max'] = $cnt;
  }

  _moduleinfo_collector_process_helper_versions($context);

  // Inform the batch engine that we are not finished,
  // and provide an estimation of the completion level we reached.
  if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
    $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
  }
}

/**
 * STEP 3
 *
 * @param $context
 */
function _moduleinfo_collector_fetch_project_page(&$context) {
  module_load_include('inc', 'moduleinfo_collector', 'includes/moduleinfo_collector.project_page');

  $fetch_settings = $_SESSION['MODULE_COLLECTOR']['settings'];

  if (!isset($context['sandbox']['progress'])) {
    $cnt = db_select('moduleinfo_collector_modules', 'mc')
      ->condition('processed', MODULEINFO_COLLECTOR_PROCESS_STEP_2)
      ->condition('status', 1)
      ->countQuery()
      ->execute()
      ->fetchField();

    $context['sandbox']['progress'] = 0;
    $context['sandbox']['current_node'] = 0;
    $context['sandbox']['max'] = $cnt;
  }

  _moduleinfo_collector_process_helper_project_page($context);

  // Inform the batch engine that we are not finished,
  // and provide an estimation of the completion level we reached.
  if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
    $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
  }
}

/**
 * STEP 4
 *
 * @param $module_name
 * @param $context
 */
function _moduleinfo_collector_modules_parse_local($module_name, $module, &$context) {
  module_load_include('inc', 'moduleinfo_collector', 'includes/moduleinfo_collector.parse_local');

  _moduleinfo_collector_process_helper_modules_parse_local($module_name, $module, $context);
}

function _moduleinfo_collector_modules_fetch_remote(&$context) {
  module_load_include('inc', 'moduleinfo_collector', 'includes/moduleinfo_collector.fetch_remote');

  if (!isset($context['sandbox']['progress'])) {
    $cnt = db_select('moduleinfo_collector_modules', 'mc')
      ->condition('processed', MODULEINFO_COLLECTOR_PROCESS_STEP_3)
      ->condition('status', 1)
      ->countQuery()
      ->execute()
      ->fetchField();

    $context['sandbox']['progress'] = 0;
    $context['sandbox']['current_node'] = 0;
    $context['sandbox']['max'] = $cnt;
  }

  _moduleinfo_collector_process_helper_modules_fetch_remote($context);

  // Inform the batch engine that we are not finished,
  // and provide an estimation of the completion level we reached.
  if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
    $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
  }
}

function _moduleinfo_collector_modules_extract(&$context) {
  module_load_include('inc', 'moduleinfo_collector', 'includes/moduleinfo_collector.modules_extract');

  if (!isset($context['sandbox']['progress'])) {
    $cnt = db_select('moduleinfo_collector_modules', 'mc')
      ->condition('processed', 163) // TODO
      ->condition('status', 1)
      ->countQuery()
      ->execute()
      ->fetchField();

    $context['sandbox']['progress'] = 0;
    $context['sandbox']['current_node'] = 0;
    $context['sandbox']['max'] = $cnt;
  }

  _moduleinfo_collector_process_helper_modules_extract($context);

  // Inform the batch engine that we are not finished,
  // and provide an estimation of the completion level we reached.
  if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
    $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
  }
}

function _moduleinfo_collector_modules_hooks(&$context) {
  module_load_include('inc', 'moduleinfo_collector', 'includes/moduleinfo_collector.modules_hooks');

  if (!isset($context['sandbox']['progress'])) {
    $cnt = db_select('moduleinfo_collector_modules', 'mc')
      ->condition('status', 1)
      ->countQuery()
      ->execute()
      ->fetchField();

    $context['sandbox']['progress'] = 0;
    $context['sandbox']['current_node'] = 0;
    $context['sandbox']['max'] = $cnt;
  }

  _moduleinfo_collector_process_helper_modules_hooks($context);

  // Inform the batch engine that we are not finished,
  // and provide an estimation of the completion level we reached.
  if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
    $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
  }
}

//function _moduleinfo_collector_modules_scan_downloads($subpath, &$context) {
//  $modules = _moduleinfo_collector_system_rebuild_module_data($subpath);
//
//  $queue = DrupalQueue::get('moduleinfo_collector');
//
//  foreach ($modules as $module) {
//    $queue->createItem($module);
//  }
//}

function _moduleinfo_collector_sanitize($text) {
  return strtr($text, 'ÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÐÑÒÓÔÕÖØÙÚÛÜÝÞßàáâãäåæçèéêëìíîïðñòóôõöøùúûýýþÿŔŕ', 'aaaaaaaceeeeiiiidnoooooouuuuybsaaaaaaaceeeeiiiidnoooooouuuyybyRr');
}

/**
 * Scans the directory structure of a given module.
 *
 * @param string $module_name
 *   Name of the module to process
 * @param array $def
 *   Definition array of the system_get_info call
 * @param array $context
 *   Context array of the Batch process
 *
 * @see system_get_info()
 */
function _moduleinfo_collector_scan($module_name, $def, &$context) {
  $files = _moduleinfo_collect_module_files($module_name);

  _moduleinfo_cache_write('files', $files, $module_name, 'moduleinfo_collector');

  return _moduleinfo_cache_operations($module_name, 'moduleinfo_collector');
}